"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bt = require("@babel/types");
var getDoclets_1 = require("../utils/getDoclets");
// tslint:disable-next-line:no-var-requires
var recast = require("recast");
function slotHandler(documentation, path) {
    if (bt.isObjectExpression(path.node)) {
        var renderPath = path
            .get('properties')
            .filter(function (p) {
            return (bt.isObjectProperty(p.node) || bt.isObjectMethod(p.node)) &&
                p.node.key.name === 'render';
        });
        // if no prop return
        if (!renderPath.length) {
            return;
        }
        var renderValuePath = bt.isObjectProperty(renderPath[0].node)
            ? renderPath[0].get('value')
            : renderPath[0];
        recast.visit(renderValuePath, {
            visitCallExpression: function (pathCall) {
                if (bt.isMemberExpression(pathCall.node.callee) &&
                    bt.isMemberExpression(pathCall.node.callee.object) &&
                    bt.isThisExpression(pathCall.node.callee.object.object) &&
                    bt.isIdentifier(pathCall.node.callee.property) &&
                    pathCall.node.callee.object.property.name === '$scopedSlots') {
                    documentation.getSlotDescriptor(pathCall.node.callee.property.name);
                    return false;
                }
                this.traverse(pathCall);
            },
            visitMemberExpression: function (pathMember) {
                if (bt.isMemberExpression(pathMember.node.object) &&
                    bt.isThisExpression(pathMember.node.object.object) &&
                    bt.isIdentifier(pathMember.node.object.property) &&
                    pathMember.node.object.property.name === '$slots' &&
                    bt.isIdentifier(pathMember.node.property)) {
                    documentation.getSlotDescriptor(pathMember.node.property.name);
                    return false;
                }
                this.traverse(pathMember);
            },
            visitJSXElement: function (pathJSX) {
                var tagName = pathJSX.node.openingElement.name;
                if (bt.isJSXIdentifier(tagName) && tagName.name === 'slot') {
                    var doc = documentation.getSlotDescriptor(getName(pathJSX));
                    doc.description = getDescription(pathJSX);
                }
                this.traverse(pathJSX);
            },
        });
    }
}
exports.default = slotHandler;
function getName(pathJSX) {
    var oe = pathJSX.node.openingElement;
    var names = oe.attributes.filter(function (a) { return bt.isJSXAttribute(a) && a.name.name === 'name'; });
    var nameNode = names.length ? names[0].value : null;
    return nameNode && bt.isStringLiteral(nameNode) ? nameNode.value : 'default';
}
function getDescription(pathJSX) {
    var siblings = pathJSX.parentPath.node.children;
    if (!siblings) {
        return '';
    }
    var indexInParent = siblings.indexOf(pathJSX.node);
    var commentExpression = null;
    for (var i = indexInParent - 1; i > -1; i--) {
        var currentNode = siblings[i];
        if (bt.isJSXExpressionContainer(currentNode)) {
            commentExpression = currentNode;
            break;
        }
    }
    if (!commentExpression || !commentExpression.expression.innerComments) {
        return '';
    }
    var cmts = commentExpression.expression.innerComments;
    var lastComment = cmts[cmts.length - 1];
    if (lastComment.type !== 'CommentBlock') {
        return '';
    }
    var docBlock = lastComment.value.replace(/^\*/, '').trim();
    var jsDoc = getDoclets_1.default(docBlock);
    if (!jsDoc.tags) {
        return '';
    }
    var slotTags = jsDoc.tags.filter(function (t) { return t.title === 'slot'; });
    if (slotTags.length) {
        var tagContent = slotTags[0].content;
        return typeof tagContent === 'string' ? tagContent : '';
    }
    return '';
}
//# sourceMappingURL=slotHandler.js.map